# module load R/3.6.1-foss-2015a-bare
# module load CDO/1.9.8-foss-2015a
# module load HDF5/1.8.14-foss-2015a

library(easyVerification)
library(startR)
library(s2dv)
library(ClimProjDiags)
library(reticulate)
library(easyNCDF)
np <- import("numpy")

archiv <- "/esarchive/oper/S2S-downscale/"

seas5 <- "/esarchive/oper/S2S-downscale/seas5/tas_Nov_DJF-1981-2020.nc"
down <- "/esarchive/scratch/llledo/downscale/downscaled.nc"
era5 <- "/esarchive/oper/S2S-downscale/era5/tas_seas_1979-2018.nc"
uerra <- "/esarchive/oper/S2S-downscale/uerra/tas_monthly-1979-2018.nc"
anlgs <- "/esarchive/oper/S2S-downscale/recons/analogs/analogs_15NN_tas_Nov_DJF-1981-2020.nc" 

#########################
# DL Models
#########################

dl_models <- c(#"rint/seas5cat_rint20x.npy",
               "rint_single/seas5cat_rintsing20x.npy",
               #"rrec/seas5cat_rrec20x.npy",
               "rspc/seas5cat_rspc20x.npy",
               "cgan_rint/seas5cat_cganrint20x.npy",
               "cgan_rint_single/seas5cat_cganrintsing20x.npy",
               "cgan_rspc/seas5cat_cganrspc20x.npy")

models_ids <- c("con","analogs","rint_s",
                "rspc","cgan_r","cgan_r_s","cgan_rspc")

dl_models <- paste0(archiv,"recons/",dl_models)
models <- NULL
n <- 0
for (model in dl_models){
   
  print(model)
  data <- np$load(model)
  names(dim(data)) <- c("sdate","time","ensemble","latitude","longitude")
  data <- MeanDims(data,'time')
  data <- Reorder(data,c("sdate","longitude","latitude","ensemble"))
  data <- InsertDim(data,posdim=1,lendim=1,name='model')

  if (n == 0){
    models <- data
  } else {
    models <- abind::abind(models,data, along=1,use.first.dimnames=T)
    names(dim(models)) <- c("model","sdate","longitude","latitude","ensemble")
  }
  n <- n + 1
}

#names(dim(models)) <- c("model", "sdate","latitude","longitude","ensemble")
models <- Reorder(models,c("sdate","longitude","latitude","ensemble","model"))

#########################
# SEAS5 
#########################

seas <- Start(dat = seas5,		  
             var = 'tas',
             time='all',
             ensemble='all',
             sdate='all',
             longitude = values(list(-0.486, 3.464)),
             latitude = values(list(40.542, 43.493)),
             longitude_reorder = CircularSort(-180, 180),
             latitude_reorder = Sort(decreasing = TRUE),
             synonims = list(latitude=c('lat','latitude'),
                             longitude=c('lon','longitude')), 
             transform = CDORemapper,
  	         transform_params = list(
  	         grid = uerra, 
  	         method = 'con',
  	         crop = c(-0.486, 3.464, 40.542, 43.493)), 
  	         transform_vars = c('latitude', 'longitude'),
	         return_vars = list(latitude = 'dat',
                                longitude = 'dat',
                                time = NULL),
	           split_multiselected_dims = TRUE,
	           retrieve = TRUE)

lons <- as.vector(attributes(seas)$Variables$dat1$longitude)
lats <- as.vector(attributes(seas)$Variables$dat1$latitude)
seas <- MeanDims(seas,'time')
seas <- Subset(seas,along=c('dat','var'),list(1,1),drop=T)
seas <- Reorder(seas,c("sdate","longitude","latitude","ensemble"))
seas <- InsertDim(seas,posdim=5,lendim=1,name='model')

#########################
# Analogs 
#########################

#hcst <- Start(dat = down,
hcst <- Start(dat = anlgs,
              var = 'tas', 
	          latitude ='all', 
	          longitude='all', 
	          ensemble='all', 
	          time='all', 
	          sdate='all',
	          longitude_reorder = CircularSort(-180, 180),
	          latitude_reorder = Sort(),
	          longitude_var = "longitude",
	          latitude_var = "latitude",
	          num_procs = 1, retrieve = T,
	          synonims = list(longitude = c('lon', 'longitude'), 
                              latitude=c('lat', 'latitude')))

lons <- as.vector(attributes(hcst)$Variables$common$longitude)
lats <- as.vector(attributes(hcst)$Variables$common$latitude)
hcst <- MeanDims(hcst,'time')
hcst <- Subset(hcst,along=c('dat','var'),list(1,1),drop=T)
hcst <- Reorder(hcst,c("sdate","longitude","latitude","ensemble"))
hcst <- InsertDim(hcst,posdim=5,lendim=1,name='model')

# merge all models
#models <- Reorder(models,c("sdate","longitude","latitude","ensemble","model"))
models <- abind::abind(hcst,models,along=5)
models <- abind::abind(seas,models,along=5)
names(dim(models)) <- c("sdate","longitude","latitude","ensemble","model")

#########################
# Observations 
#########################

obs <- Start(dat = uerra,
             var = 'tas', 
	         longitude = values(list(-0.486, 3.464)),
	         latitude = values(list(40.542, 43.493)),
	         time='all', 
	         longitude_reorder = CircularSort(-180, 180),
	         latitude_reorder = Sort(),
	         longitude_var = "longitude",
	         latitude_var = "latitude",
	         num_procs = 1, retrieve = T,
	         synonims = list(longitude = c('lon', 'longitude'), 
                             latitude=c('lat', 'latitude')),
		     return_vars = list(	time = NULL,
					latitude = 'dat',
					longitude = 'dat')
	)


lons2 <- as.vector(attributes(obs)$Variables$dat1$longitude)
lats2 <- as.vector(attributes(obs)$Variables$dat1$latitude)
obs <- Season(obs,time_dim='time',1,12,2)
obs <- Subset(obs,along=c('dat','var'),list(1,1),drop=T)
names(dim(obs)) <- c("sdate","longitude","latitude")
obs <- InsertDim(obs,posdim=4,lendim=1,name='model')

obs.mm <- obs
for(dat in 1:(dim(models)['model'][[1]]-1)){
    obs.mm <- abind::abind(obs.mm,obs,
                           along=which(names(dim(obs)) == 'model'))
}
names(dim(obs.mm)) <- names(dim(obs)) 
obs <- obs.mm
remove(obs.mm)

#########################
# Verification 
#########################

hyears <- 1981:2020
oyears <- 1979:2017 

verif.years <- intersect(hyears,oyears)

obs <- Subset(obs,along='sdate',oyears %in% verif.years)
models <- Subset(models,along='sdate',oyears %in% verif.years)

# skill scores
bias <- Apply(Apply(models,target_dims='ensemble',
                    na.rm=T,fun=mean)[[1]] - obs,
              target_dims='sdate', na.rm=T,fun=mean)[[1]]

rpss <- veriApply("EnsRpss", fcst=models, obs=obs, prob= 1:2/3, ensdim=4, tdim=1, na.rm=T)
corr <- veriApply("EnsCorr", fcst=models, obs=obs, ensdim=4, tdim=1, na.rm=T)

corr <- Reorder(corr,c("model","longitude","latitude"))
bias <- Reorder(bias,c("model","longitude","latitude"))
rpss <- Reorder(rpss$skillscore,c("model","longitude","latitude"))
metrics <- abind::abind(rpss,corr,bias, along=0)
names(dim(metrics)) <- c("metric","model","longitude","latitude")
metrics <- Reorder(metrics,c("model","metric","longitude","latitude"))

dim(lons2) <- list(longitude=length(lons2))
dim(lats2) <- list(latitude=length(lats2))
models_ids <- 1:7
dim(models_ids) <- list(model=length(models_ids))
metrics_ids <- 1:3
dim(metrics_ids) <- list(metric=length(metrics_ids))

ArrayToNc(list(metrics,lons2,lats2,
               models_ids,metrics_ids),
          'test3.nc')
